#!/usr/bin/perl 
#------------------------------------------------------------
# script for Rob Lee - parse a Registry hive file, pulling keys
# and sorting based on LastWrite time.
#
# copyright 2008 H. Carvey
#------------------------------------------------------------
use strict;
use Parse::Win32Registry qw(:REG_);
use Getopt::Long;
# Included to permit compiling via Perl2Exe
#perl2exe_include "Parse/Win32Registry/Key.pm";
#perl2exe_include "Parse/Win32Registry/Value.pm";
my $VERSION = "1\.0\.20081021";

my %config = ();
Getopt::Long::Configure("prefix_pattern=(-|\/)");
GetOptions(\%config, qw(m=s reg|r=s ini|i=s help|?|h));

if ($config{help} || !%config || !$config{reg}) {
	\usage();
	exit 0;
}

$config{m} = "\./" unless $config{m};
$config{m} =~ s/\\/\//g;
#$config{m} = $config{m}."/" unless ($config{m} =~ m/\/$/);
die $config{reg}." not found.\n" unless (-e $config{reg});

my %regkeys;
my $reg = Parse::Win32Registry->new($config{reg});
my $root_key = $reg->get_root_key;

traverse($root_key);

foreach my $t (reverse sort {$a <=> $b} keys %regkeys) {
	foreach my $item (@{$regkeys{$t}}) {
		$item =~ s/\\/\//g;
		print "0|".$config{m}."$item|0|0|0|0|0|0|$t|0|0\n";
	}
}

sub traverse {
	my $key = shift;
  my $ts = $key->get_timestamp();
  my $name = $key->as_string();
  $name =~ s/\$\$\$PROTO\.HIV//;
  $name = (split(/\[/,$name))[0];
  push(@{$regkeys{$ts}},$name);  
	foreach my $subkey ($key->get_list_of_subkeys()) {
		traverse($subkey);
  }
}

sub usage {
    print << "USAGE";
regtime v. $VERSION 
Traverse through a Registry hive file, listing all keys and their LastWrite
times.  Output is displayed sorted by most recent time first, and is suitable
for use with TSK v3.0 fls and mactime

  -m hive.........Hive file to prepend to key paths (use / separator, use _ 
                  or enclose in quotes if spaces in path)
  -r hive.........Hive file to parse
  -h..............Help (print this information)
  
Ex: C:\>regtime -m HKEY_USER -r NTUSER.DAT
    C:\>regtime -m HKLM/System -r system    
    
copyright 2008 H. Carvey    
USAGE
}
  
