#!/usr/bin/perl

#######################################################################################################################
# Purpose: Process regp.pl output of the system registry file for current control set and timezone information
#
# By: Keven Murphy (Citadel Systems, L.L.C.)
# URL: http://www.citadelsystems.net
#
# Notes: Regp.pl can be found at http://sourceforge.net/projects/windowsir/ and is written by Harlan Carvey.
#
# Version: 0.1
#
#######################################################################################################################

use DateTime::Format::WindowsFileTime;
use DateTime::TimeZone;
use DateTime;
use integer;
use Getopt::Long;
use Switch;

$result = GetOptions ("file=s"   => \$file,      # string
			"verbose"  => \$verbose);  # flag
#######################################################################################################################
#
#	Useful Information
#
#
# To find which Control set to use
# Look at Current
#\$$$PROTO.HIV\Select
#LastWrite time: Sat Dec  3 20:59:37 2005
#        --> Current;REG_DWORD;2
#        --> Default;REG_DWORD;2
#        --> Failed;REG_DWORD;1
#        --> LastKnownGood;REG_DWORD;4
#	
#	
#\$$$PROTO.HIV\ControlSet002\Control\Windows
#LastWrite time: Mon Mar 26 12:47:24 2007
#        --> CSDVersion;REG_DWORD;512
#        --> CSDReleaseType;REG_DWORD;0
#        --> Directory;REG_EXPAND_SZ;%SystemRoot%^@
#        --> ErrorMode;REG_DWORD;0
#        --> NoInteractiveServices;REG_DWORD;0
#        --> SystemDirectory;REG_EXPAND_SZ;%SystemRoot%\system32^@
#        --> ShellErrorMode;REG_DWORD;1
#        --> ShutdownTime;REG_BINARY;04 74 46 e7 a4 6f c7 01
#	
#\$$$PROTO.HIV\ControlSet002\Control\TimeZoneInformation
#LastWrite time: Mon Mar 26 13:31:43 2007
#        --> Bias;REG_DWORD;300
#        --> StandardName;REG_SZ;Eastern Standard Time^@
#        --> StandardBias;REG_DWORD;0
#        --> StandardStart;REG_BINARY;00 00 0b 00 01 00 02 00 00 00 00 00 00 00 00 00
#        --> DaylightName;REG_SZ;Eastern Daylight Time^@
#        --> DaylightBias;REG_DWORD;4294967236
#        --> DaylightStart;REG_BINARY;00 00 03 00 02 00 02 00 00 00 00 00 00 00 00 00
#        --> ActiveTimeBias;REG_DWORD;240
#	
#	
#	
# TimeZone info in software
# Microsoft\Windows NT\CurrentVersion\Time Zones\Eastern Standard Time
#
#
######################################################################################################################


#Definations
@monthlist=("Error on Month","January","February","March","April","May","June","July","August","September","October","November","December");
@weekth=("Error on Month","st","nd","rd","th","th");
@weekday=("Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday");

#Create array of timezones
@timezonelist = DateTime::TimeZone->all_names;

#Set time to Epoch to help determine timezone
$dt = DateTime->new( 	year  => 1970,
			month => 1,
			day   => 1,
			hour  => 00,
			minute => 00,
			second => 00 );

#Time Starts 
sub timestarts {
        my $timeline = shift;
	($year2,$year1,$month2,$month1,$week2,$week1,$hour2,$hour1,$min2,$min1,$sec2,$sec1,$msec2,$msec1,$day2,$day1) = split(/ /, $timeline);
	$year= hex($year1.$year2);
	$month=hex($month1.$month2);
	$week=hex($week1.$week2);
	$hour=hex($hour1.$hour2);
	$min=hex($min1.$min2);
	$sec=hex($sec1.$sec2);
	$msec=hex($msec1.$msec2);
	$day=hex($day1.$day2);
	print "Starts: On $weekday[$day] the $week$weekth[$week] week of $monthlist[$month] at ";
	printf "%02u:%02u:%02u:%02u", $hour,$min,$sec,$msec;
	if ($hour < 11) {
		print " (AM)\n";
	   } else {
   		print " (PM)\n";
	}
}

#List possible timezones to use in Sleuthkit
sub FindTimeZone {
	print "Possible TimeZone matches: \n";
	foreach $timezone (@timezonelist) {
		my $tz = DateTime::TimeZone->new( name => $timezone );
		my $offsetinseconds = $tz->offset_for_datetime($dt);
		$offsetinhours = ($offsetinseconds / 60) / 60;
		if ( $offsetinhours == $biasinhours ) {
		    if ($bias == $activetimebias ) {
		         if (! $tz->has_dst_changes) {
			    print "\t$timezone\tOffset: $offsetinhours\n";
			 }
		    } else {
			  if ($tz->has_dst_changes) {
			     print "\t$timezone\t\t\tStandard Offset: $offsetinhours\tDaylight Savings Offset: ". ($offsetinhours +1) ."\n";
			  }
		    }
	        }
	}
}

#Figure out MSB related to TimeZone Bias
sub timezonebias {
     my $bias = shift;
     $bin = sprintf "%b",$bias;
     if ( substr($bin,0,1) == 1) {
          $bias = - $bias;
	  #printf "~ value is %#034b  %d  %x\n", $bias, $bias, $bias;
	  $return = sprintf "%d",$bias;
	  #print "Return: $return\n";
	  return $return; 
        } else {
	  $return = sprintf "%d",$bias;
	  return $return; 
     }

}

sub WindowsShutdownTime {
 	my $shutdowntimebinary = shift;
	$shutdowntimebinaryrev = join(" ", reverse split(" ", $shutdowntimebinary));
	$shutdowntimebinaryrev=~ s/ //g;

	$shutdowntime = DateTime::Format::WindowsFileTime->parse_datetime(  $shutdowntimebinaryrev );
	return $shutdowntime;
}

sub GetControlSet {
    my @controlset=("> Current;REG_DWORD;","> Default;REG_DWORD;","> Failed;REG_DWORD;","> LastKnownGood;REG_DWORD;" );

    for ($i = 0; $i <= 3; $i++) {
	    my $line = `grep -a "$controlset[$i]" $file`;
	    chomp ($line);
	    
	    switch ($i) {
	           case 0 	{ $currentcontrolset=&processline($line) }
		   case 1	{ $defaultcontrolset=&processline($line) }
		   case 2       { $failedcontrolset=&processline($line) }
		   case 3       { $lastknowngoodcontrolset=&processline($line) }
	    }
    } 
}

sub processline {
    my $line = shift;
    @words = split(/;/, $line);
    return $words[2];
}



sub FindControlSet {
   my $currentcontrolset = shift;
   my $foundcontrol = 0;
   my $foundtimezone = 0;
   my $shutdowntime = "";
   my @words;
   
   open F, $file or print "\n* Couldn't open ${file}\n\n" && return;
   $ControlWindowsStr = "ControlSet00".$currentcontrolset."ControlWindows";
   $ShutdownTimeStr = "ShutdownTime;REG_BINARY;";
   $TimeZoneStr = "ControlSet00".$currentcontrolset."ControlTimeZoneInformation";
   
    while (<F>) {  
       $line = $_;  
       chomp($line);
       $line =~ s/\\//g;

       if ($foundtimezone == 0) {
	       if ($line =~ /($TimeZoneStr)/o) { 
                  $foundtimezone++;
	       }   
       } else {
       	       if ($foundtimezone == 1) {
	           switch ($line) {
                	case /--> Bias;REG_DWORD;/o            { $bias=&processline($line) }
                	case /StandardName;REG_SZ;/o           { $standardname=&processline($line) }
                	case /StandardBias;REG_DWORD;/o        { $standardbias=&processline($line) }
                	case /StandardStart;REG_BINARY;/o      { $standardstart=&processline($line) }
                	case /DaylightName;REG_SZ;/o           { $daylightname=&processline($line) }
			case /DaylightBias;REG_DWORD;/o        { $daylightbias=&processline($line) }
			case /DaylightStart;REG_BINARY;/o      { $daylightstart=&processline($line) }
			case /--> ActiveTimeBias;REG_DWORD;/o  { $activetimebias=&processline($line) }
                	else                                   { $foundtime++ }
                    }	  
	       }
       }      
       if ($foundcontrol == 0) {
	      if ($line =~ /($ControlWindowsStr)/o) {            # /o = compile regex
		 $foundcontrol++;
	      }
       } else {
	      if ($line =~ /($ShutdownTimeStr)/o) {            # /o = compile regex
		 $shutdowntimebinary=&processline($line);
		 last;
	      } 
       }        
    }
    close F;
}


GetControlSet();
print "Current Control Set: $currentcontrolset\n";
print "Default Control Set: $defaultcontrolset\n";
print "Failed Control Set: $failedcontrolset\n";
print "Last Known Good Control Set: $lastknowngoodcontrolset\n";

FindControlSet($currentcontrolset);


$biasinhours = (&timezonebias($bias)) / 60;
$activetimebiasinhours = (&timezonebias($activetimebias)) / 60;


print "\n\nStandard Time Zone: $standardname\n";
print "Standard Hours: $biasinhours\n";
print "Standard Time ";
timestarts ($standardstart);
print "\n";
print "Day Light Savings Zone Name: $daylightname\n";
if ($bias == $activetimebias ) {
	print "Day Light Saving: None\n";
    } else {
        print "Day Light Saving: ". ($biasinhours+1) ."\n";
}
print "Daylight Savings ";
timestarts ($daylightstart);
print "\nCurrent Time Zone offset: ". $activetimebiasinhours . "\n";

print "\nShutdown Time: ". &WindowsShutdownTime($shutdowntimebinary) . "\n\n";  # yields '2005-01-15T03:00:00'

FindTimeZone();
